//
// DONE
//
// DemoViewController.m
// WTFeedbackViewDemo   1.0
//
// Created by Wagner Truppel on 2009-10-08.
// Copyright 2009 Wagner Truppel (jrcapab@gmail.com). All rights reserved.
//
// Redistribution and use in binary and source forms, with or without
// modification, are permitted for any project, commercial or otherwise,
// provided that the following conditions are met:
//
// Redistributions in binary form must display the copyright notice in the
// About view, website, and/or documentation.
//
// Redistributions of source code must retain the copyright notice, this
// list of conditions, and the following disclaimer.
//
// THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THIS SOFTWARE.

#import "CustomLogging.h"
#import "DemoViewController.h"
#import "WTFeedbackView.h"
#import "FunkyFeedbackView.h"
#import "Enums.h"



// ========================================================================= //
                       #pragma mark string constants
// ========================================================================= //

static NSString* const kText1 = @"Reading Data...";
static NSString* const kText2 = @"Encrypting & Archiving...";
static NSString* const kText3 = @"Saving...";

// ========================================================================= //
                       #pragma mark private interface
// ========================================================================= //

@interface DemoViewController ()

@property (readwrite, nonatomic, retain) UITextField* textField;
@property (readwrite, nonatomic, retain) UIButton* showAgainBtn;

@property (readwrite, nonatomic, retain) UISwitch* coverNavBarSwitch;
@property (readwrite, nonatomic, retain) UILabel*  coverNavBarLabel;

@property (readwrite, nonatomic, retain) UISwitch* showKeyboardSwitch;
@property (readwrite, nonatomic, retain) UILabel*  showKeyboardLabel;

@property (readwrite, nonatomic, retain) UISwitch* progressFirstSwitch;
@property (readwrite, nonatomic, retain) UILabel*  progressFirstLabel;

@property (readwrite, nonatomic, retain) UILabel* staticStyleLabel;
@property (readwrite, nonatomic, retain) UILabel*       styleLabel;

@property (readwrite, nonatomic, retain) UILabel*  staticKindLabel;
@property (readwrite, nonatomic, retain) UILabel*        kindLabel;

@property (readwrite, nonatomic, retain) UILabel* contentLabel;

@property (readwrite, nonatomic, retain) UILabel* method1Label;
@property (readwrite, nonatomic, retain) UILabel* method2Label;
@property (readwrite, nonatomic, retain) UILabel* method3Label;

@property (readwrite, nonatomic, retain) UILabel* setTextLabel;

@property (readwrite, nonatomic, retain) NSTimer* progressTimer;

@end

// ========================================================================= //

@interface DemoViewController (Private)

- (void) setControlStates;

- (void) updateControlStatesGoingIn;
- (void) updateControlStatesGoingOut;

- (void)   changeFromText1ToText2;
- (void) doChangeFromText1ToText2;

- (void)   changeFromText2ToText3;
- (void) doChangeFromText2ToText3;

- (void) doDisplayFeedbackView;

- (void) removeFeedbackView;
- (void) doRemoveFeedbackView;

- (void) updateProgress;
- (void) fireUpdateProgress: (NSTimer*) timer;

- (void) switchViews;

- (void) switchToProgress;
- (void) doSwitchToProgress;
- (void) finallySwitchToProgress;

- (void) switchToActivity;
- (void) doSwitchToActivity;
- (void) finallySwitchToActivity;

@end

// ========================================================================= //



@implementation DemoViewController

// ========================================================================= //
                           #pragma mark properties
// ========================================================================= //

@synthesize textField;
@synthesize showAgainBtn;

@synthesize coverNavBarSwitch;
@synthesize coverNavBarLabel;

@synthesize showKeyboardSwitch;
@synthesize showKeyboardLabel;

@synthesize progressFirstSwitch;
@synthesize progressFirstLabel;

@synthesize staticStyleLabel;
@synthesize       styleLabel;

@synthesize  staticKindLabel;
@synthesize        kindLabel;

@synthesize contentLabel;

@synthesize method1Label;
@synthesize method2Label;
@synthesize method3Label;

@synthesize setTextLabel;

@synthesize   styleText;
@synthesize    kindText;

@synthesize contentText;

@synthesize method1Text;
@synthesize method2Text;
@synthesize method3Text;

@synthesize progressTimer;

@synthesize section;
@synthesize row;

// ========================================================================= //
                            #pragma mark methods
// ========================================================================= //

- (void) dealloc;
{
    BEGIN
    // WTLOG(@"DemoViewController: -dealloc");

    [   textField release];
    [showAgainBtn release];

    [coverNavBarSwitch release];
    [coverNavBarLabel  release];

    [showKeyboardSwitch release];
    [showKeyboardLabel  release];

    [progressFirstSwitch release];
    [progressFirstLabel  release];

    [staticStyleLabel release];
    [      styleLabel release];
    [      styleText  release];

    [ staticKindLabel release];
    [       kindLabel release];
    [       kindText  release];

    [contentLabel release];
    [contentText  release];

    [method1Label release];
    [method1Text  release];

    [method2Label release];
    [method2Text  release];

    [method3Label release];
    [method3Text  release];

    [setTextLabel release];

    [super dealloc];

    END
}

// ========================================================================= //

- (void) viewDidLoad;
{
    BEGIN
    // WTLOG(@"DemoViewController: -viewDidLoad");

    [super viewDidLoad];

    self.view.backgroundColor = [UIColor groupTableViewBackgroundColor];

    staticStyleLabel.font = [UIFont     systemFontOfSize: 13];
          styleLabel.font = [UIFont boldSystemFontOfSize: 15];

     staticKindLabel.font = [UIFont     systemFontOfSize: 13];
           kindLabel.font = [UIFont boldSystemFontOfSize: 15];

        contentLabel.font = [UIFont boldSystemFontOfSize: 13];
        setTextLabel.font = [UIFont     systemFontOfSize: 13];

        method1Label.font = [UIFont     systemFontOfSize: 13];
        method2Label.font = [UIFont     systemFontOfSize: 13];
        method3Label.font = [UIFont     systemFontOfSize: 13];

       styleLabel.text = styleText;
        kindLabel.text = kindText;

     contentLabel.text = contentText;
     setTextLabel.text = nil;

     method1Label.text = method1Text;
     method2Label.text = method2Text;
     method3Label.text = method3Text;

      coverNavBar =   coverNavBarSwitch.on;
     showKeyboard =  showKeyboardSwitch.on;
    progressFirst = progressFirstSwitch.on;

    [self setControlStates];
    [self displayFeedbackView];

    END
}

// ========================================================================= //

- (void) viewDidDisappear: (BOOL) animated;
{
    BEGIN
    if (animated)
    {
        // WTLOG(@"DemoViewController: -viewDidDisappear: animated = YES");
    }
    else
    {
        // WTLOG(@"DemoViewController: -viewDidDisappear: animated = NO");
    }

    [self doRemoveFeedbackView];
    [super viewDidDisappear: animated];

    END
}

// ========================================================================= //

- (BOOL) shouldAutorotateToInterfaceOrientation:
            (UIInterfaceOrientation) interfaceOrientation;
{
    BEGIN
    // WTLOG(@"DemoViewController: -shouldAutorotateToInterfaceOrientation");
    RETURN YES;
    END
}

// ========================================================================= //
                           #pragma mark IB actions
// ========================================================================= //

- (IBAction) toggleCoverNavBar;
{
    BEGIN
    // WTLOG(@"DemoViewController: -toggleCoverNavBar");

    coverNavBar = coverNavBarSwitch.on;
    [self displayFeedbackView];

    END
}

// ========================================================================= //

- (IBAction) toggleShowProgressViewFirst;
{
    BEGIN
    // WTLOG(@"DemoViewController: -toggleShowProgressViewFirst");

    progressFirst = progressFirstSwitch.on;
    [self displayFeedbackView];

    END
}

// ========================================================================= //

- (IBAction) toggleShowKeyboard;
{
    BEGIN
    // WTLOG(@"DemoViewController: -toggleShowKeyboard");

    showKeyboard = showKeyboardSwitch.on;
    [self displayFeedbackView];

    END
}

// ========================================================================= //

- (IBAction) displayFeedbackView;
{
    BEGIN
    // WTLOG(@"DemoViewController: -displayFeedbackView");

    [self updateControlStatesGoingIn];

    END
}

// ========================================================================= //
                        #pragma mark private methods
// ========================================================================= //

- (void) setControlStates;
{
    BEGIN
    // WTLOG(@"DemoViewController: -setControlStates");

    [textField resignFirstResponder];
    textField.hidden = YES;

    if (section == kSecSimpleActivity)
    {
          coverNavBarSwitch.hidden = YES;
           coverNavBarLabel.hidden = YES;

         showKeyboardSwitch.hidden = YES;
          showKeyboardLabel.hidden = YES;

        progressFirstSwitch.hidden = YES;
         progressFirstLabel.hidden = YES;
    }
    else
    if (section == kSecBezelActivity ||
        section == kSecBezelProgress)
    {
          coverNavBarSwitch.hidden = NO;
           coverNavBarLabel.hidden = NO;

         showKeyboardSwitch.hidden = NO;
          showKeyboardLabel.hidden = NO;

        progressFirstSwitch.hidden = YES;
         progressFirstLabel.hidden = YES;
    }
    else
    if (section == kSecBezelFlexible)
    {
          coverNavBarSwitch.hidden = NO;
           coverNavBarLabel.hidden = NO;

         showKeyboardSwitch.hidden = NO;
          showKeyboardLabel.hidden = NO;

        progressFirstSwitch.hidden = NO;
         progressFirstLabel.hidden = NO;
    }
    else
    if (section == kSecKeyboardActivity ||
        section == kSecKeyboardProgress)
    {
        showKeyboard = YES;

          coverNavBarSwitch.hidden = YES;
           coverNavBarLabel.hidden = YES;

         showKeyboardSwitch.hidden = YES;
          showKeyboardLabel.hidden = YES;

        progressFirstSwitch.hidden = YES;
         progressFirstLabel.hidden = YES;
    }
    else
    if (section == kSecKeyboardFlexible)
    {
        showKeyboard = YES;

          coverNavBarSwitch.hidden = YES;
           coverNavBarLabel.hidden = YES;

         showKeyboardSwitch.hidden = YES;
          showKeyboardLabel.hidden = YES;

        progressFirstSwitch.hidden = NO;
         progressFirstLabel.hidden = NO;
    }

    END
}

// ========================================================================= //

- (void) updateControlStatesGoingIn;
{
    BEGIN
    // WTLOG(@"DemoViewController: -updateControlStatesGoingIn");

    if (showKeyboard)
    {
               showAgainBtn.enabled = NO;
          coverNavBarSwitch.enabled = NO;
         showKeyboardSwitch.enabled = NO;
        progressFirstSwitch.enabled = NO;

        [textField becomeFirstResponder];
    }
    else
    {
        [textField resignFirstResponder];

               showAgainBtn.enabled = YES;
          coverNavBarSwitch.enabled = YES;
         showKeyboardSwitch.enabled = YES;
        progressFirstSwitch.enabled = YES;
    }

    [self performSelector: @selector(doDisplayFeedbackView)
               withObject: nil
               afterDelay: 1.0];

    END
}

// ========================================================================= //

- (void) updateControlStatesGoingOut;
{
    BEGIN
    // WTLOG(@"DemoViewController: -updateControlStatesGoingOut");

           showAgainBtn.enabled = YES;
      coverNavBarSwitch.enabled = YES;
     showKeyboardSwitch.enabled = YES;
    progressFirstSwitch.enabled = YES;

    [textField resignFirstResponder];

    [[self class] cancelPreviousPerformRequestsWithTarget: self];

    END
}

// ========================================================================= //

- (void) changeFromText1ToText2;
{
    BEGIN
    // WTLOG(@"DemoViewController: -changeFromText1ToText2");

    NSString* str = [[NSString alloc] initWithFormat:
        @"[WTFeedbackView setText: @\"%@\"];", kText2];
    setTextLabel.text = str;
    [str release];

    [self performSelector: @selector(doChangeFromText1ToText2)
               withObject: nil
               afterDelay: 1.5];

    END
}

// ========================================================================= //

- (void) doChangeFromText1ToText2;
{
    BEGIN
    // WTLOG(@"DemoViewController: -doChangeFromText1ToText2");

    [WTFeedbackView setText: kText2];

    if (section == kSecBezelProgress    || section == kSecBezelFlexible ||
        section == kSecKeyboardProgress || section == kSecKeyboardFlexible)
    { [self updateProgress]; }

    [self performSelector: @selector(changeFromText2ToText3)
               withObject: nil
               afterDelay: 3.0];

    END
}

// ========================================================================= //

- (void) changeFromText2ToText3;
{
    BEGIN
    // WTLOG(@"DemoViewController: -changeFromText2ToText3");

    NSString* str = [[NSString alloc] initWithFormat:
        @"[WTFeedbackView setText: @\"%@\"];", kText3];
    setTextLabel.text = str;
    [str release];

    [self performSelector: @selector(doChangeFromText2ToText3)
               withObject: nil
               afterDelay: 1.5];

    END
}

// ========================================================================= //

- (void) doChangeFromText2ToText3;
{
    BEGIN
    // WTLOG(@"DemoViewController: -doChangeFromText2ToText3");

    [WTFeedbackView setText: kText3];

    if (section == kSecBezelProgress    || section == kSecBezelFlexible ||
        section == kSecKeyboardProgress || section == kSecKeyboardFlexible)
    { [self updateProgress]; }

    [self performSelector: @selector(removeFeedbackView)
               withObject: nil
               afterDelay: 3.0];

    END
}

// ========================================================================= //

- (void) doDisplayFeedbackView;
{
    BEGIN
    // WTLOG(@"DemoViewController: -displayFeedbackView");

    UIView* viewToUse = self.view;

    if (coverNavBar)
    { viewToUse = self.navigationController.navigationBar.superview; }

    BOOL changeText = NO;

    switch (section)
    {
        default:
        case kSecSimpleActivity:
            switch (row)
            {
                default:
                case kSecSimpleActivityVFV:
                {
                    [WTFeedbackView newSimpleActivityViewForView: viewToUse];
                }
                break;

                case kSecSimpleActivityVFV_WT:
                {
                    [WTFeedbackView newSimpleActivityViewForView: viewToUse
                                                        withText: kText1];
                    changeText = YES;
                }
                break;
            }
            break;

        case kSecBezelActivity:
            switch (row)
            {
                default:
                case kSecBezelActivityVFV:
                {
                    [WTFeedbackView newBezelActivityViewForView: viewToUse];
                }
                break;

                case kSecBezelActivityVFV_WT:
                {
                    [WTFeedbackView newBezelActivityViewForView: viewToUse
                                                       withText: kText1];
                    changeText = YES;
                }
                break;
            }
            break;

        case kSecBezelProgress:
            switch (row)
            {
                default:
                case kSecBezelProgressVFV:
                {
                    [WTFeedbackView newBezelProgressViewForView: viewToUse];
                    [self updateProgress];
                }
                break;

                case kSecBezelProgressVFV_WT:
                {
                    [WTFeedbackView newBezelProgressViewForView: viewToUse
                                                       withText: kText1];
                    [self updateProgress];
                    changeText = YES;
                }
                break;
            }
            break;

        case kSecBezelFlexible:
            switch (row)
            {
                default:
                case kSecBezelFlexibleVFV:
                {
                    [WTFeedbackView newBezelFlexibleViewForView: viewToUse
                                            showingProgressView: progressFirst];
                    [self switchViews];
                    RETURN;
                }
                break;

                case kSecBezelFlexibleVFV_WT:
                {
                    [WTFeedbackView newBezelFlexibleViewForView: viewToUse
                                            showingProgressView: progressFirst
                                                       withText: kText1];
                    [self switchViews];
                    RETURN;
                }
                break;
            }
            break;

        case kSecKeyboardActivity:
            switch (row)
            {
                default:
                case kSecKeyboardActivityVFV:
                {
                    [WTFeedbackView newKeyboardActivityView];
                }
                break;

                case kSecKeyboardActivityVFV_WT:
                {
                    [WTFeedbackView newKeyboardActivityViewWithText: kText1];
                    changeText = YES;
                }
                break;
            }
            break;

        case kSecKeyboardProgress:
            switch (row)
            {
                default:
                case kSecKeyboardProgressVFV:
                {
                    [WTFeedbackView newKeyboardProgressView];
                    [self updateProgress];
                }
                break;

                case kSecKeyboardProgressVFV_WT:
                {
                    [WTFeedbackView newKeyboardProgressViewWithText: kText1];
                    [self updateProgress];
                    changeText = YES;
                }
                break;
            }
            break;

        case kSecKeyboardFlexible:
            switch (row)
            {
                default:
                case kSecKeyboardFlexibleVFV:
                {
    [WTFeedbackView newKeyboardFlexibleViewShowingProgressView: progressFirst];
                    [self switchViews];
                    RETURN;
                }
                break;

                case kSecKeyboardFlexibleVFV_WT:
                {
    [WTFeedbackView newKeyboardFlexibleViewShowingProgressView: progressFirst
                                                      withText: kText1];
                    [self switchViews];
                    RETURN;
                }
                break;
            }
            break;

        case kSecFunkyFeedbackView:
            switch (row)
            {
                default:
                case kSecFunkyFeedbackViewVFV_WT:
                {
                    [FunkyFeedbackView newFunkyViewForView: viewToUse
                                       showingProgressView: progressFirst
                                                  withText: kText1];
                    [self switchViews];
                    RETURN;
                }
                break;
            }
            break;
    }

    if (changeText)
    {
        [self performSelector: @selector(changeFromText1ToText2)
                   withObject: nil
                   afterDelay: 3.0];
    }
    else
    {
        [self performSelector: @selector(removeFeedbackView)
                   withObject: nil
                   afterDelay: 3.0];
    }

    END
}

// ========================================================================= //

- (void) removeFeedbackView;
{
    BEGIN
    // WTLOG(@"DemoViewController: -removeFeedbackView");

    if (section == kSecSimpleActivity)
    { setTextLabel.text = @"[WTFeedbackView removeView];"; }
    else
    { setTextLabel.text = @"[WTFeedbackView removeViewAnimating: YES];"; }

    [self performSelector: @selector(doRemoveFeedbackView)
               withObject: nil
               afterDelay: 1.5];

    END
}

// ========================================================================= //

- (void) doRemoveFeedbackView;
{
    BEGIN
    // WTLOG(@"DemoViewController: -doRemoveFeedbackView");

    setTextLabel.text = nil;
    [self.progressTimer invalidate];

    if (section == kSecSimpleActivity)
    { [WTFeedbackView removeView]; }
    else
    { [WTFeedbackView removeViewAnimating: YES]; }

    [self performSelector: @selector(updateControlStatesGoingOut)
               withObject: nil
               afterDelay: 0.5];

    END
}

// ========================================================================= //

- (void) updateProgress;
{
    BEGIN
    // WTLOG(@"DemoViewController: -updateProgress");

    counter = 0;

    [self.progressTimer invalidate];
     self.progressTimer = [NSTimer scheduledTimerWithTimeInterval: 0.04
        target: self selector: @selector(fireUpdateProgress:)
        userInfo: nil repeats: YES];

    END
}

// ========================================================================= //

- (void) fireUpdateProgress: (NSTimer*) timer;
{
    BEGIN
    // WTLOG(@"DemoViewController: -timerFireMethod: %@", timer);

    [WTFeedbackView updateProgress: (counter / 100.0)];
    counter += 1;

    if (counter > 100)
    { [timer invalidate]; }

    END
}

// ========================================================================= //

- (void) switchViews;
{
    BEGIN
    // WTLOG(@"DemoViewController: -switchViews");

    if (progressFirst)
    {
        [self updateProgress];
        [self switchToActivity];
    }
    else
    { [self switchToProgress]; }

    END
}

// ========================================================================= //

- (void) switchToProgress;
{
    BEGIN
    // WTLOG(@"DemoViewController: -switchToProgress");

    [self performSelector: @selector(doSwitchToProgress)
               withObject: nil
               afterDelay: 3.0];

    END
}

// ========================================================================= //

- (void) doSwitchToProgress;
{
    BEGIN
    // WTLOG(@"DemoViewController: -doSwitchToProgress");

    setTextLabel.text = @"[WTFeedbackView switchToProgressView];";

    [self performSelector: @selector(finallySwitchToProgress)
               withObject: nil
               afterDelay: 1.5];

    END
}

// ========================================================================= //

- (void) finallySwitchToProgress;
{
    BEGIN
    // WTLOG(@"DemoViewController: -finallySwitchToProgress");

    setTextLabel.text = nil;
    [WTFeedbackView switchToProgressView];
    [self updateProgress];

    [self performSelector: @selector(changeFromText2ToText3)
               withObject: nil
               afterDelay: 3.0];

    END
}

// ========================================================================= //

- (void) switchToActivity;
{
    BEGIN
    // WTLOG(@"DemoViewController: -switchToActivity");

    [self performSelector: @selector(doSwitchToActivity)
               withObject: nil
               afterDelay: 3.0];

    END
}

// ========================================================================= //

- (void) doSwitchToActivity;
{
    BEGIN
    // WTLOG(@"DemoViewController: -doSwitchToActivity");

    setTextLabel.text = @"[WTFeedbackView switchToActivityIndicatorView];";

    [self performSelector: @selector(finallySwitchToActivity)
               withObject: nil
               afterDelay: 1.5];

    END
}

// ========================================================================= //

- (void) finallySwitchToActivity;
{
    BEGIN
    // WTLOG(@"DemoViewController: -finallySwitchToActivity");

    setTextLabel.text = nil;
    [WTFeedbackView switchToActivityIndicatorView];

    [self performSelector: @selector(changeFromText2ToText3)
               withObject: nil
               afterDelay: 3.0];

    END
}

// ========================================================================= //

@end
